package com.coolimagingproject.basicImageProcessingOperations.operador.panelesOperadores;

import com.coolimagingproject.coolimaging.libreriaimagenes.operador.publico.ConjuntoParametroOperador;
import com.coolimagingproject.coolimaging.libreriaimagenes.operador.publico.ParametroOperador;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.coolimagingproject.basicImageProcessingOperations.operador.panelesOperadores.utilidades.SelectorMetodoInterpolacion;
import com.coolimagingproject.basicImageProcessingOperations.operador.panelesOperadores.utilidades.SelectorMetodoTratamientoBorde;
import com.coolimagingproject.basicImageProcessingOperations.operador.tratamientoImagenes.OperadorRotacion;
import com.coolimagingproject.basicImageProcessingOperations.operador.panelesOperadores.utilidades.ConstantesGlobales;
import com.coolimagingproject.coolimaging.vista.operador.panelesOperadores.publico.PanelInfoOperador;


/**
 * <p>
 * La clase PanelInfoOperadorRotacion representa un panel de informacin (hereda
 * de PanelInfoOperador) que almacena la informacin relativa a un operador de
 * rotacin de imgenes (OperadorRotacion), a saber:
 * 
 * <ul>
 * <li>ngulo de rotacin.
 * <li>Centro de rotacin (coordenadas X e Y respectivamente).
 * <li>Mtodo de interpolacin usado para corregir los defectos provocados por
 * la rotacin de la imagen.
 * <li>Mtodo de tratamiento del borde de la imagen para los pxeles que no
 * estn definidos.
 * </ul>
 * 
 * @author L.A Gonzlez Jaime
 * @author R.J Palma Durn
 * 
 */
public class PanelInfoOperadorRotacion extends PanelInfoOperador{
	// Etiqueta del ngulo de rotacin
	protected Label etiquetaAnguloRotacion;
	// Etiqueta del centro de rotacin del eje X
	protected Label etiquetaCentroRotacionX;
	// Etiqueta del centro de rotacin del eje Y
	protected Label etiquetaCentroRotacionY;
	// Etiqueta del mtodo de interpolacin
	protected Label etiquetaMetodoInterpolacion;
	// Etiqueta del mtodo de extensin del borde
	protected Label etiquetaMetodoBorde;
	// Texto del ngulo de rotacin
	protected Text textoAnguloRotacion;
	// Texto del centro de rotacin del eje X
	protected Text textoCentroRotacionX;
	// Texto del centro de rotacin del eje Y
	protected Text textoCentroRotacionY;
	// Lista de los mtodos de interpolacin a poder usar en la rotacin
	protected SelectorMetodoInterpolacion listaMetodosInterpolacion;
	// Lista de los mtodos de tratamiento del borde de la imagen en la rotacin
	protected SelectorMetodoTratamientoBorde listaMetodosBorde;
	// Botn que permite indicar si la imagen es centrada tras la rotacin
	protected Button botonCentrarImagen;

	/**
	 * Construye un objeto de tipo PanelInfoOperadorRotacion con un estilo
	 * determinado, y emplazado en un Composite padre.
	 * 
	 * @param padre
	 *        Composite padre en el que se situar este
	 *        PanelInfoOperadorRotacion.
	 * @param estilo
	 *        estilo del PanelInfoOperadorEscalado.
	 */
	public PanelInfoOperadorRotacion(Composite padre, int estilo){
		super(padre, estilo);
		this.setLayout(new FillLayout());
		// Grupo con ttulo que almacena los componentes y les pone un borde con
		// ttulo
		Group grupoGlobal;
		// Grupo con ttulo que almacena las componentes del centro de rotacin
		Group grupoCentroRotacion;
		// Inicializacin del Group que engloba todo
		grupoGlobal = new Group(this, SWT.NONE);
		grupoGlobal.setText("Parmetros de la rotacin");
		// Declaracin y asignacn del layout
		GridLayout layout = new GridLayout(2, false);
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		grupoGlobal.setLayout(layout);
		// Declaracin de componentes
		this.etiquetaAnguloRotacion = new Label(grupoGlobal, SWT.NONE);
		this.textoAnguloRotacion = new Text(grupoGlobal, SWT.BORDER);
		grupoCentroRotacion = new Group(grupoGlobal, SWT.NONE);
		this.etiquetaCentroRotacionX = new Label(grupoCentroRotacion, SWT.NONE);
		this.textoCentroRotacionX = new Text(grupoCentroRotacion, SWT.BORDER);
		this.etiquetaCentroRotacionY = new Label(grupoCentroRotacion, SWT.NONE);
		this.textoCentroRotacionY = new Text(grupoCentroRotacion, SWT.BORDER);
		this.etiquetaMetodoInterpolacion = new Label(grupoGlobal, SWT.NONE);
		this.listaMetodosInterpolacion = new SelectorMetodoInterpolacion(
				grupoGlobal, SWT.NONE);
		this.etiquetaMetodoBorde = new Label(grupoGlobal, SWT.NONE);
		this.listaMetodosBorde = new SelectorMetodoTratamientoBorde(
				grupoGlobal, SWT.NONE);
		this.botonCentrarImagen = new Button(grupoGlobal, SWT.CHECK);
		this.botonCentrarImagen.setSelection(true);
		// Configuracin de componentes
		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.horizontalSpan = 2;
		grupoCentroRotacion.setText("Centro de rotacin");
		grupoCentroRotacion.setLayout(layout);
		grupoCentroRotacion.setLayoutData(data);
		this.etiquetaAnguloRotacion.setText("ngulo de rotacin");
		this.etiquetaCentroRotacionX.setText("Centro X");
		this.etiquetaCentroRotacionY.setText("Centro Y");
		this.etiquetaMetodoInterpolacion.setText("Mtodo de interpolacin");
		this.etiquetaMetodoBorde.setText("Mtodo de extensin del borde");
		this.etiquetaAnguloRotacion
				.setToolTipText("ngulo de rotacin de la imagen, medido en grados, y en sentido horario");
		this.etiquetaCentroRotacionX
				.setToolTipText("Coordenada X del pxel usado como centro de rotacin");
		this.etiquetaCentroRotacionY
				.setToolTipText("Coorndeada Y del pxel usado como centro de rotacin");
		this.etiquetaMetodoInterpolacion
				.setToolTipText("Mtodo de interpolacin usado para corregir los defectos producidos por la rotacin");
		this.etiquetaMetodoBorde
				.setToolTipText("Mtodo de tratamiento del borde de la imagen para los pxeles no definidos");
		this.botonCentrarImagen.setText("Centrar imagen");
		this.botonCentrarImagen
				.setToolTipText("Determina si la imagen es centrada tras la rotacin");
		this.listaMetodosInterpolacion
				.setToolTipText("Mtodo de interpolacin usado para corregir los defectos producidos  por la rotacin");
		this.listaMetodosBorde
				.setToolTipText("Mtodo de tratamiento del borde de la imagen para los pxeles no definidos");
		this.textoAnguloRotacion.setText("0");
		this.textoCentroRotacionX.setText("0");
		this.textoCentroRotacionY.setText("0");
		this.textoAnguloRotacion
				.setToolTipText("ngulo de rotacin de la imagen, medido en grados, y en sentido horario");
		this.textoCentroRotacionX
				.setToolTipText("Coordenada X del pxel usado como centro de rotacin");
		this.textoCentroRotacionY
				.setToolTipText("Coorndeada Y del pxel usado como centro de rotacin");
		data = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		data.widthHint = ConstantesGlobales.ANCHO_PREDETERMINADO_CAMPO_TEXTO;
		this.textoAnguloRotacion.setLayoutData(data);
		data = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		data.widthHint = ConstantesGlobales.ANCHO_PREDETERMINADO_CAMPO_TEXTO;
		this.textoCentroRotacionX.setLayoutData(data);
		data = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		data.widthHint = ConstantesGlobales.ANCHO_PREDETERMINADO_CAMPO_TEXTO;
		this.textoCentroRotacionY.setLayoutData(data);
		data = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		data.widthHint = ConstantesGlobales.ANCHO_PREDETERMINADO_LISTA_DESPLEGABLE;
		this.listaMetodosBorde.setLayoutData(data);
		data = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		data.widthHint = ConstantesGlobales.ANCHO_PREDETERMINADO_LISTA_DESPLEGABLE;
		this.listaMetodosInterpolacion.setLayoutData(data);
		data = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		data.horizontalSpan = 2;
		this.botonCentrarImagen.setLayoutData(data);
		this.pack();
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * utilidades.PanelInfoOperador#getParametros(utilidades.ParametrosOperador)
	 */
	public void getParametros(ConjuntoParametroOperador parametros){
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_ANGULO_ROTACION, new Float(
						this.textoAnguloRotacion.getText())
						* new Float(Math.PI) / 180.0f));
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_CENTRO_ROTACION_X, new Float(
						this.textoCentroRotacionX.getText())));
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_CENTRO_ROTACION_Y, new Float(
						this.textoCentroRotacionY.getText())));
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_METODO_INTERPOLACION,
				this.listaMetodosInterpolacion.getInterpolacion()));
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_METODO_BORDE, this.listaMetodosBorde
						.getTratadorBorde()));
		parametros.insertarParametro(new ParametroOperador(
				OperadorRotacion.PARAMETRO_ROTAR_Y_CENTRAR,
				this.botonCentrarImagen.getSelection()));
	}

	/*
	 * (non-Javadoc)
	 * @see utilidades.ValidadorDatos#validar(java.lang.StringBuffer)
	 */
	public boolean validar(StringBuffer mensajesError){
		try{
			Float.parseFloat(this.textoAnguloRotacion.getText());
		}
		catch(Exception e){
			mensajesError
					.append(new String(
							"El ngulo de rotacin especificado no es un nmero real vlido"));
			return false;
		}
		try{
			if(Float.parseFloat(this.textoCentroRotacionX.getText()) < 0){
				mensajesError
						.append(new String(
								"La coordenada X del centro de rotacin no puede ser negativa"));
				return false;
			}
		}
		catch(Exception e){
			mensajesError
					.append(new String(
							"La coordenada X del centro de rotacin especificada no es un nmero real vlido"));
			return false;
		}
		try{
			if(Float.parseFloat(this.textoCentroRotacionY.getText()) < 0){
				mensajesError
						.append(new String(
								"La coordenada Y del centro de rotacin no puede ser negativa"));
				return false;
			}
		}
		catch(Exception e){
			mensajesError
					.append(new String(
							"La coordenada Y del centro de rotacin especificada no es un nmero real vlido"));
			return false;
		}
		return true;
	}
}
